import os
from copy import deepcopy
from pynwb import load_namespaces, get_class, register_class


from hdmf.common import DynamicTable
from hdmf.utils import docval, call_docval_func, get_docval, getargs, popargs


# Set path of the namespace.yaml file to the expected install location
ndx_nirs_specpath = os.path.join(
    os.path.dirname(__file__), "spec", "ndx-nirs.namespace.yaml"
)

# If the extension has not been installed yet but we are running directly from
# the git repo
if not os.path.exists(ndx_nirs_specpath):
    ndx_nirs_specpath = os.path.abspath(
        os.path.join(
            os.path.dirname(__file__),
            "..",
            "..",
            "..",
            "spec",
            "ndx-nirs.namespace.yaml",
        )
    )

# Load the namespace
load_namespaces(ndx_nirs_specpath)

# TODO: import your classes here or define your class using get_class to make
# them accessible at the package level


def update_docval(overridden_fn, **kwargs):
    """Copy the docval from an existing function and update specified parameters"""
    original_docval = get_docval(overridden_fn)
    new_docval = deepcopy(original_docval)
    for name, update_vals in kwargs.items():
        for item in new_docval:
            if item["name"] == name:
                item.update(update_vals)
                break
        else:
            msg = "docval item named {} does not exist for function {}".format(
                name, overridden_fn.__name__
            )
            raise ValueError(msg)
    return new_docval


sources_docval = update_docval(
    DynamicTable.__init__,
    name={"default": "sources"},
    description={"default": "A table describing optical sources of a NIRS device"},
)


@register_class("NIRSSourcesTable", "ndx-nirs")
class NIRSSourcesTable(DynamicTable):
    """A DynamicTable representing the optical sources of this NIRS device"""

    __columns__ = (
        dict(name="label", description="The label of the optical source"),
        dict(name="x", description="The x coordinate of the optical source"),
        dict(name="y", description="The y coordinate of the optical source"),
        dict(
            name="z",
            description="The z coordinate of the optical source",
            required=False,
        ),
    )

    @docval(*sources_docval)
    def __init__(self, **kwargs):
        call_docval_func(super().__init__, kwargs)


detectors_docval = update_docval(
    DynamicTable.__init__,
    name={"default": "detectors"},
    description={"default": "A table describing optical detectors of a NIRS device"},
)


@register_class("NIRSDetectorsTable", "ndx-nirs")
class NIRSDetectorsTable(DynamicTable):
    """A DynamicTable representing the optical detectors of this NIRS device"""

    __columns__ = (
        dict(name="label", description="The label of the optical detector"),
        dict(name="x", description="The x coordinate of the optical detector"),
        dict(name="y", description="The y coordinate of the optical detector"),
        dict(
            name="z",
            description="The z coordinate of the optical detector",
            required=False,
        ),
    )

    @docval(*detectors_docval)
    def __init__(self, **kwargs):
        call_docval_func(super().__init__, kwargs)


channels_docval = [
    {
        "name": "sources",
        "type": NIRSSourcesTable,
        "doc": "The table that is referenced by the source column",
        "default": None,
    },
    {
        "name": "detectors",
        "type": NIRSDetectorsTable,
        "doc": "The table that is referenced by the detector column",
        "default": None,
    },
    *update_docval(
        DynamicTable.__init__,
        name={"default": "channels"},
        description={"default": "A table describing optical channels of a NIRS device"},
    ),
]


@register_class("NIRSChannelsTable", "ndx-nirs")
class NIRSChannelsTable(DynamicTable):
    """A DynamicTable representing the optical channels of this NIRS device"""

    __columns__ = (
        dict(name="label", description="The label of the channel", required=True),
        dict(
            name="source",
            description="A reference to the optical source for this channel in NIRSSourcesTable",
            required=True,
            table=True,
        ),
        dict(
            name="detector",
            description="A reference to the optical detector for this channel in NIRSDetectorsTable",
            required=True,
            table=True,
        ),
        dict(
            name="source_wavelength",
            description="The wavelength of light in nm emitted by the source for this channel.",
            required=True,
        ),
        dict(
            name="emission_wavelength",
            description=(
                "The wavelength of light in nm emitted by the fluorophore under "
                "fluorescent spectroscopy for this channel. Only used for fluorescent"
                " spectroscopy"
            ),
            required=False,
        ),
        dict(
            name="source_power",
            description="The power of the source in mW used for this channel.",
            required=False,
        ),
        dict(
            name="detector_gain",
            description="The gain applied to the detector for this channel.",
            required=False,
        ),
    )

    @docval(*channels_docval)
    def __init__(self, **kwargs):
        sources = popargs("sources", kwargs)
        detectors = popargs("detectors", kwargs)
        call_docval_func(super().__init__, kwargs)
        self.source.table = sources
        self.detector.table = detectors

    @docval(
        {
            "name": "sources",
            "type": NIRSSourcesTable,
            "doc": "The table that is referenced by the source column",
        }
    )
    def set_sources_table(self, **kwargs):
        """Assigns the NIRSourcesTable which should be referenced by the source column"""
        sources = getargs("sources", kwargs)
        self.source.table = sources

    @docval(
        {
            "name": "detectors",
            "type": NIRSDetectorsTable,
            "doc": "The table that is referenced by the detector column",
        }
    )
    def set_detectors_table(self, **kwargs):
        """Assigns the NIRDetectorsTable which should be referenced by the detector column"""
        detectors = getargs("detectors", kwargs)
        self.detector.table = detectors


NIRSDevice = get_class("NIRSDevice", "ndx-nirs")
NIRSSeries = get_class("NIRSSeries", "ndx-nirs")



import pytest
import pandas as pd
import numpy as np

from pynwb.testing import TestCase
from hdmf.utils import get_docval
from hdmf.common import DynamicTable, DynamicTableRegion

from ndx_nirs import (
    NIRSSourcesTable,
    NIRSDetectorsTable,
    NIRSChannelsTable,
    NIRSDevice,
    NIRSSeries,
    update_docval,
)


def test_update_docval_does_not_modify_original_docval():
    """Verify that update_docval does not modify the original fn docval"""
    original_docval = get_docval(DynamicTable.__init__)
    _ = update_docval(DynamicTable.__init__, name={"default": "FOO"})
    assert "default" not in original_docval[0]


def test_update_docval_correctly_updates_fields():
    """Verify that update_docval correctly updates the new docval"""
    new_docval = update_docval(
        DynamicTable.__init__,
        name={"default": "FOO", "miscfield": "xyz"},
        description={"required": False},
    )
    assert new_docval[0]["default"] == "FOO"
    assert new_docval[0]["miscfield"] == "xyz"
    assert not new_docval[1]["required"]


def test_update_docval_raises_error_if_parameter_does_not_exist():
    """Verify that update_docval raises a ValueError if the parameter doesn't exist"""
    with pytest.raises(ValueError):
        _ = update_docval(
            DynamicTable.__init__,
            foo={"default": "bar"},
        )


class TestNIRSSourcesTable(TestCase):
    """Unit tests for NIRSSourcesTable"""

    def test_empty_constructor(self):
        """Verify that the table can be instantiated with default name and description"""
        table = NIRSSourcesTable()

        self.assertEqual(table.name, "sources")
        self.assertIsInstance(table.description, str)

    def test_add_row_without_optional_z(self):
        """Verify that add_row correctly adds a row when z is omitted"""
        table = NIRSSourcesTable()
        table.add_row(label="foo", x=1.0, y=2.0)

        self.assertEqual(len(table), 1)
        self.assertEqual(table.label[0], "foo")
        self.assertEqual(table.x[0], 1.0)
        self.assertEqual(table.y[0], 2.0)
        self.assertIsNone(table.z)

    def test_add_row_with_optional_z(self):
        """Verify that add_row correctly adds a row when z is included"""
        table = NIRSSourcesTable()
        table.add_row(label="foo", x=1.0, y=2.0, z=3.0)

        self.assertEqual(len(table), 1)
        self.assertEqual(table.label[0], "foo")
        self.assertEqual(table.x[0], 1.0)
        self.assertEqual(table.y[0], 2.0)
        self.assertEqual(table.z[0], 3.0)


class TestNIRSDetectorsTable(TestCase):
    """Unit tests for NIRSDetectorsTable"""

    def test_empty_constructor(self):
        """Verify that the table can be instantiated with default name and description"""
        table = NIRSDetectorsTable()

        self.assertEqual(table.name, "detectors")
        self.assertIsInstance(table.description, str)

    def test_add_row_without_optional_z(self):
        """Verify that add_row correctly adds a row when z is omitted"""
        table = NIRSDetectorsTable()
        table.add_row(label="foo", x=1.0, y=2.0)

        self.assertEqual(len(table), 1)
        self.assertEqual(table.label[0], "foo")
        self.assertEqual(table.x[0], 1.0)
        self.assertEqual(table.y[0], 2.0)
        self.assertIsNone(table.z)

    def test_add_row_with_optional_z(self):
        """Verify that add_row correctly adds a row when z is included"""
        table = NIRSDetectorsTable()
        table.add_row(label="foo", x=1.0, y=2.0, z=3.0)

        self.assertEqual(len(table), 1)
        self.assertEqual(table.label[0], "foo")
        self.assertEqual(table.x[0], 1.0)
        self.assertEqual(table.y[0], 2.0)
        self.assertEqual(table.z[0], 3.0)


def create_fake_sources_table():
    """Returns a NIRSSourcesTable which can be used for testing"""
    table = NIRSSourcesTable()
    for n in range(7):
        table.add_row({"label": f"S{n+1}", "x": (n % 4) - 1.5, "y": (n % 3) - 1.0})
    return table


def create_fake_detectors_table():
    """Returns a NIRSDetectorsTable which can be used for testing"""
    table = NIRSDetectorsTable()
    for n in range(4):
        table.add_row({"label": f"D{n+1}", "x": (n % 2) + 0.5, "y": n - 3.5})
    return table


def create_fake_channels_table():
    """Returns a NIRSChannelsTable which can be used for testing"""
    sources = create_fake_sources_table()
    detectors = create_fake_detectors_table()
    source_detector_pairs = [(n, n % 5) for n in range(7)]
    table = NIRSChannelsTable(sources, detectors)
    ch_id = 0
    for source_idx, detector_idx in source_detector_pairs:
        for wavelength in [690.0, 830.0]:
            table.add_row(
                label=f"CH{ch_id}",
                source=source_idx,
                detector=detector_idx,
                source_wavelength=wavelength,
            )
            ch_id += 1
    return table


class TestNIRSChannelsTable(TestCase):
    """Unit tests for NIRSChannelsTable"""

    def test_constructor_assigns_tables(self):
        """Verify that the table can be instantiated and assigns source and detector tables"""
        sources = create_fake_sources_table()
        detectors = create_fake_detectors_table()
        table = NIRSChannelsTable(sources, detectors)

        self.assertEqual(table.name, "channels")
        self.assertIsInstance(table.description, str)
        self.assertIs(table.source.table, sources)
        self.assertIs(table.detector.table, detectors)

    def test_set_sources_table(self):
        """Verify that the sources table can be set after instantiation"""
        table = NIRSChannelsTable()
        sources = create_fake_sources_table()
        table.set_sources_table(sources)
        self.assertIs(table.source.table, sources)

    def test_set_detectors_table(self):
        """Verify that the sources table can be set after instantiation"""
        table = NIRSChannelsTable()
        detectors = create_fake_detectors_table()
        table.set_detectors_table(detectors)
        self.assertIs(table.detector.table, detectors)

    def test_add_row_with_the_correct_columns_provided(self):
        """Verify that add_row correctly adds a row when the correct columns are provided"""
        table = NIRSChannelsTable(
            create_fake_sources_table(), create_fake_detectors_table()
        )
        table.add_row(label="foo", source=6, detector=1, source_wavelength=123.5)

        self.assertEqual(len(table), 1)
        self.assertEqual(table.label[0], "foo")
        expected_source = pd.DataFrame(
            dict(label="S7", x=0.5, y=-1.0), index=pd.Index([6], name="id")
        )
        pd.testing.assert_frame_equal(table.source[0], expected_source)
        expected_detector = pd.DataFrame(
            dict(label="D2", x=1.5, y=-2.5), index=pd.Index([1], name="id")
        )
        pd.testing.assert_frame_equal(table.detector[0], expected_detector)
        self.assertEqual(table.source_wavelength[0], 123.5)

    def test_add_row_with_optional_columns(self):
        """Verify that optional columns can be specified"""
        table = NIRSChannelsTable(
            create_fake_sources_table(), create_fake_detectors_table()
        )
        table.add_row(
            label="foo",
            source=6,
            detector=1,
            source_wavelength=123.5,
            emission_wavelength=234.6,
            source_power=11.0,
            detector_gain=5.1,
        )

        self.assertEqual(table.emission_wavelength[0], 234.6)
        self.assertEqual(table.source_power[0], 11.0)
        self.assertEqual(table.detector_gain[0], 5.1)


class TestNIRSDevice(TestCase):
    """Unit tests for NIRSDevice"""

    def test_initialization(self):
        """Verify that fields are correctly defined after initialization"""
        channels = create_fake_channels_table()
        device = NIRSDevice(
            name="test_device",
            description="Foo",
            nirs_mode="time-domain",
            channels=channels,
            sources=channels.source.table,
            detectors=channels.detector.table,
        )
        self.assertEqual(device.name, "test_device")
        self.assertIs(device.channels, channels)
        self.assertIs(device.sources, channels.source.table)
        self.assertIs(device.detectors, channels.detector.table)

    def create_test_device_with_params(self, **kwargs):
        """Returns an initialized NIRSDevice giving it additional kwargs"""
        channels = create_fake_channels_table()
        return NIRSDevice(
            name="test_device",
            description="Foo",
            nirs_mode="time-domain",
            channels=channels,
            sources=channels.source.table,
            detectors=channels.detector.table,
            **kwargs,
        )

    def test_optional_attributes(self):
        """Verify that all expected optional attributes can be provided to a NIRSDevice"""
        test_attributes = {
            "frequency": 30.5,
            "time_delay": 25.0,
            "time_delay_width": 13.0,
            "correlation_time_delay": 0.013,
            "correlation_time_delay_width": 0.008,
            "additional_parameters": "some additional text",
        }
        for name, value in test_attributes.items():
            with self.subTest(attr_name=name, attr_value=value):
                device = self.create_test_device_with_params(**{name: value})
                self.assertEqual(
                    getattr(device, name),
                    value,
                    f"failed for '{name}' attribute subtest",
                )


class TestNIRSSeries(TestCase):
    """Unit tests for NIRSSeries"""

    def test_initialization(self):
        """Verify that fields are correctly defined after initialization"""
        channels = create_fake_channels_table()
        fake_timeseries = np.arange(0, 100, 0.05)  # in seconds
        fake_data = np.random.rand(len(fake_timeseries), len(channels))
        series = NIRSSeries(
            name="nirs_data",
            description="The raw NIRS channel data",
            timestamps=fake_timeseries,
            channels=DynamicTableRegion(
                name="channels",
                description="an ordered map to the channels in this NIRS series",
                table=channels,
                data=channels.id[:],
            ),
            data=fake_data,
            unit="V",
        )

        self.assertEqual(series.name, "nirs_data")
        np.testing.assert_array_equal(series.timestamps[:], fake_timeseries)
        np.testing.assert_array_equal(series.data[:], fake_data)
        self.assertIs(series.channels.table, channels)
        self.assertEqual(series.unit, "V")



import datetime
import tempfile
from os import path

import numpy as np

from pynwb import NWBHDF5IO
from pynwb.file import NWBFile, Subject
from pynwb.testing import TestCase, remove_test_file
from hdmf.common import DynamicTableRegion

from ndx_nirs import (
    NIRSDevice,
    NIRSSeries,
    NIRSSourcesTable,
    NIRSDetectorsTable,
    NIRSChannelsTable,
)


def setup_nwbfile():
    nwbfile = NWBFile(
        session_description="A test NWB NIRS file",
        identifier="nirs_test",
        session_start_time=datetime.datetime(
            2021, 4, 1, 15, 0, 0, tzinfo=datetime.timezone.utc
        ),
        subject=Subject(subject_id="X"),
    )

    nirs_device = create_fake_nirs_device()
    nwbfile.add_device(nirs_device)
    nwbfile.add_acquisition(create_fake_nirs_series(nirs_device))
    return nwbfile


def create_fake_sources_table():
    table = NIRSSourcesTable()
    table.add_row({"label": "S1", "x": 0.0, "y": -1.0})
    table.add_row({"label": "S2", "x": 0.0, "y": 1.0})
    return table


def create_fake_detectors_table():
    table = NIRSDetectorsTable()
    table.add_row({"label": "D1", "x": 0.0, "y": -2.0})
    table.add_row({"label": "D2", "x": 0.0, "y": 0.0})
    table.add_row({"label": "D3", "x": 0.0, "y": 2.0})
    return table


def create_fake_channels_table():
    sources = create_fake_sources_table()
    detectors = create_fake_detectors_table()
    source_detector_pairs = [(0, 0), (0, 1), (1, 1), (1, 2)]
    table = NIRSChannelsTable(sources, detectors)
    ch_id = 0
    for source_idx, detector_idx in source_detector_pairs:
        for wavelength in [690.0, 830.0]:
            table.add_row(
                label=f"CH{ch_id}",
                source=source_idx,
                detector=detector_idx,
                source_wavelength=wavelength,
            )
            ch_id += 1
    return table


def create_fake_nirs_device():
    channels = create_fake_channels_table()

    return NIRSDevice(
        name="device",
        description="An fNIRS device",
        manufacturer="XYZ",
        nirs_mode="time-domain",
        channels=channels,
        sources=channels.source.table,
        detectors=channels.detector.table,
        time_delay=4.2,
        time_delay_width=0.5,
        additional_parameters="flux_capacitor_gain = 9000; speaker_volume = 11;",
    )


def create_fake_nirs_series(device):
    channels = device.channels
    fake_timeseries = np.arange(0, 100, 0.05)  # in seconds
    fake_data = np.random.rand(len(fake_timeseries), len(channels))

    return NIRSSeries(
        name="nirs_data",
        description="The raw NIRS channel data",
        timestamps=fake_timeseries,
        channels=DynamicTableRegion(
            name="channels",
            description="an ordered map to the channels in this NIRS series",
            table=channels,
            data=channels.id[:],
        ),
        data=fake_data,
        unit="V",
    )


class NIRSIntegrationTests(TestCase):
    """Integration tests for the ndx-nirs extension classes"""

    def setUp(self):
        self.nwb = setup_nwbfile()
        self.path = path.join(tempfile.gettempdir(), "test.nwb")

    def tearDown(self):
        remove_test_file(self.path)

    def test_roundtrip(self):
        """Verify that an NWBFile with NIRS data saved to disk and read again is
        identical to the original NWBFile
        """

        with NWBHDF5IO(self.path, "w") as io:
            io.write(self.nwb)

        with NWBHDF5IO(self.path, "r") as io:
            read_nwb = io.read()
            self.assertContainerEqual(self.nwb, read_nwb)
            device = read_nwb.devices["device"]
            self.assertIs(device.channels.source.table, device.sources)
            self.assertIs(device.channels.detector.table, device.detectors)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

project = "ndx-nirs"
copyright = "2021, Sumner L Norman,Darin Erat Sleiter,José Ribeiro"
author = "Sumner L Norman,Darin Erat Sleiter,José Ribeiro"

# The short X.Y version
version = "0.2.0"

# The full version, including alpha/beta/rc tags
release = "0.2.0"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.ifconfig",
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "English"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["*.ipynb_checkpoints*"]


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "alabaster"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ["_static"]
html_static_path = []


# -- Extension configuration -------------------------------------------------

# -- Options for intersphinx extension ---------------------------------------
############################################################################
#  CUSTOM CONFIGURATIONS ADDED BY THE NWB TOOL FOR GENERATING FORMAT DOCS
###########################################################################

import sphinx_rtd_theme  # noqa: E402
import textwrap  # noqa: E402

# -- Options for intersphinx  ---------------------------------------------
intersphinx_mapping = {"core": ("https://nwb-schema.readthedocs.io/en/latest/", None)}

# -- Generate sources from YAML---------------------------------------------------
# Always rebuild the source docs from YAML even if the folder with the source files already exists
spec_doc_rebuild_always = True


def run_doc_autogen(_):
    # Execute the autogeneration of Sphinx format docs from the YAML sources
    import sys
    import os

    conf_file_dir = os.path.dirname(os.path.abspath(__file__))
    sys.path.append(
        conf_file_dir
    )  # Need so that generate format docs can find the conf_doc_autogen file
    from conf_doc_autogen import spec_output_dir

    if spec_doc_rebuild_always or not os.path.exists(spec_output_dir):
        sys.path.append("./docs")  # needed to enable import of generate_format docs
        from hdmf_docutils.generate_format_docs import main as generate_docs

        generate_docs()


def setup(app):
    app.connect("builder-inited", run_doc_autogen)
    app.add_css_file("theme_overrides.css")  # overrides for wide tables in RTD theme


# -- Customize sphinx settings
numfig = True
autoclass_content = "both"
autodoc_docstring_signature = True
autodoc_member_order = "bysource"
add_function_parentheses = False


# -- HTML sphinx options
html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# LaTeX Sphinx options
latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    "papersize": "letterpaper",
    # The font size ('10pt', '11pt' or '12pt').
    "pointsize": "10pt",
    # Additional stuff for the LaTeX preamble.
    "preamble": textwrap.dedent(
        """
        \\setcounter{tocdepth}{3}
        \\setcounter{secnumdepth}{6}
        \\usepackage{enumitem}
        \\setlistdepth{100}
        """
    ),
}



# -*- coding: utf-8 -*-

import os

from setuptools import setup, find_packages
from shutil import copy2

# load README.md/README.rst file
try:
    if os.path.exists("README.md"):
        with open("README.md", "r") as fp:
            readme = fp.read()
            readme_type = "text/markdown; charset=UTF-8"
    elif os.path.exists("README.rst"):
        with open("README.rst", "r") as fp:
            readme = fp.read()
            readme_type = "text/x-rst; charset=UTF-8"
    else:
        readme = ""
except Exception:
    readme = ""

setup_args = {
    "name": "ndx-nirs",
    "version": "0.2.0",
    "description": "An NWB extension for storing Near-Infrared Spectroscopy (NIRS) data",
    "long_description": readme,
    "long_description_content_type": readme_type,
    "author": "Sumner L Norman,Darin Erat Sleiter,José Ribeiro",
    "author_email": "sumner@ae.studio,darin@ae.studio,jose@ae.studio",
    "url": "https://github.com/agencyenterprise/ndx-nirs",
    "license": "BSD 3-Clause",
    "python_requires": "~=3.6",
    "install_requires": ["hdmf>=2.5.6,<3", "pynwb>=1.5.1,<2"],
    "packages": find_packages("src/pynwb"),
    "package_dir": {"": "src/pynwb"},
    "package_data": {
        "ndx_nirs": [
            "spec/ndx-nirs.namespace.yaml",
            "spec/ndx-nirs.extensions.yaml",
        ]
    },
    "classifiers": [
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
    ],
    "zip_safe": False,
}


def _copy_spec_files(project_dir):
    ns_path = os.path.join(project_dir, "spec", "ndx-nirs.namespace.yaml")
    ext_path = os.path.join(project_dir, "spec", "ndx-nirs.extensions.yaml")

    dst_dir = os.path.join(project_dir, "src", "pynwb", "ndx_nirs", "spec")
    if not os.path.exists(dst_dir):
        os.mkdir(dst_dir)

    copy2(ns_path, dst_dir)
    copy2(ext_path, dst_dir)


if __name__ == "__main__":
    _copy_spec_files(os.path.dirname(__file__))
    setup(**setup_args)
